@@ -1,7 +1,7 @@
 # -*- encoding: utf-8 -*-
-# vim: expandtab:tabstop=4:softtabstop=4:autoindent
+# vim: expandtab:tabstop=4:shiftwidth=4:softtabstop=4:autoindent
 
-"""nmevent v0.2.1 - C#-like implementation of the Observer pattern
+"""nmevent v0.3.1 - C#-like implementation of the Observer pattern
 
 This is a Python module :mod:`nmevent`, simple C#-like implementation of
 the Observer pattern (http://en.wikipedia.org/wiki/Observer_pattern).
@@ -18,12 +18,17 @@ The most straightfoward way to use :mod:`nmevent` is this:
 >>> class ExampleClass(object):
 ...    def __init__(self):
 ...       self.event = nmevent.Event()
-...
-...    def _do_something(self):
+... 
+...    def do_something(self):
 ...       self.event(self)
 ...
+>>> def handler(sender, **keywords):
+...    print "event occured"
+...
 >>> example = ExampleClass()
 >>> example.event += handler
+>>> example.do_something()
+event occured
 
 It should be noted, that event doesn't necessarily need to be an object
 attribute. :class:`Event` instance is basically just a callable object that
@@ -33,7 +38,7 @@ This usage, however, isn't very C#-like. In C#, events are declared in class
 scope and that's why the :class:`Event` class also supports the descriptor
 protocol (you can use the same way you use the built-in ``property`` object).
 
->>> from nmevent import EventSlot
+>>> from nmevent import Event
 >>> class ExampleClass(object):
 ...    event = Event()
 ...
@@ -60,7 +65,6 @@ instances of :class:`Property` class. It also creates events for the built-in
 ``property`` objects, but you have to raise the events yourself in the setter
 function or elsewhere.
 
->>> import nmevent
 >>> @nmevent.with_events
 ... class ExampleClass(object):
 ...    @nmevent.nmproperty
@@ -79,10 +83,18 @@ function or elsewhere.
 ...    def y(self, value):
 ...       old_value, self._y = self._y, value
 ...       self.y_changed(old_value = old_value)
+... 
+...    def __init__(self):
+...       self._x = None
+...       self._y = None
+...
+>>> def handler(sender, **keywords):
+...    print "x changed"
 ...
 >>> example = ExampleClass()
 >>> example.x_changed += handler
 >>> example.x = 10 # handler gets called
+x changed
 
 =======
 License
@@ -123,6 +135,20 @@ v0.2.1
   Rewritten some unit tests and added new ones. Improved documentation
   a little bit.
 
+v0.3
+  Fixed a major bug, which caused an unbound event not to be actually
+  bound when called with an object instance as the first argument.
+
+  Added the :func:`with_properties` class decorator, which automatically
+  decorates a class with "private" attributes for each property and
+  automatic getters and setters where either one of them is missing.
+
+v0.3.1
+  Added docstring tests and fixed all the docstrings so that they
+  would pass. As a result, another problem was found with the event
+  binding. That problem was fixed by adding the :meth:`InstanceEvent.bind`
+  method to be used mainly by the :class:`Property` class when raising
+  the "changed" events.
 """
 
 __version__ = __doc__.splitlines()[0].split(' ')[1][1:]
@@ -130,6 +156,7 @@ __author__ = u"Jan Milik <milikjan@fit.cvut.cz>"
 __all__    = [
     'nmproperty',
     'with_events',
+    'with_properties',
     'Event',
 ]
 
@@ -221,7 +248,7 @@ class Event(object):
 
     def __get__(self, obj, objtype = None):
         return self.bind(objtype, obj)
-
+    
     def __set__(self, obj, value):
         # raise AttributeError, "Events are read-only attributes."
         pass
@@ -237,10 +264,10 @@ class Event(object):
         """Adds a handler (observer) to this event.
 
         ``__iadd__`` attribute of this class is just an alias of this
-        method, so the two following statements are equivalent:
-
-        >>> event.add_handler(handler)
-        >>> event += handler
+        method, so the last two of the following statements are equivalent:
+        
+        >>> event.add_handler(handler) # doctest: +SKIP
+        >>> event += handler # doctest: +SKIP
         """
         self.handlers.add(handler)
         return self
@@ -313,7 +340,7 @@ class InstanceEvent(object):
 
        The following condition must be always true:
 
-       >>> isinstance(self.im_sender, self.im_class)
+       >>> isinstance(self.im_sender, self.im_class) # doctest: +SKIP
     """
     
     __slots__ = ('im_event', 'im_class', 'im_sender', )
@@ -351,8 +378,9 @@ class InstanceEvent(object):
                         (self.im_class.__name__))
             sender = args[0]
             args = args[1:]
+            return self.im_event.bind(self.im_class, sender)(*args, **keywords)
         return self.handlers(sender, *args, **keywords)
-
+    
     def __iadd__(self, handler):
         if self.is_bound:
             self.handlers.add(handler)
@@ -381,6 +409,19 @@ class InstanceEvent(object):
             return "<bound event>"
         return "<unbound event>"
     
+    def bind(self, objtype, obj):
+        """Attempts to bind the instance event to an instance.
+
+        Note that this method silently fails and returns self
+        when the event is already bound. This is by design
+        and is meant to unify the :class:`Event`'s and 
+        :class:`InstanceEvent`'s protocol.
+        """
+        
+        if self.is_bound:
+            return self
+        return InstanceEvent(self.im_event, self.im_class, obj)
+
 class Property(object):
     """Eventful property descriptor.
 
@@ -399,17 +440,27 @@ class Property(object):
     Usage:
 
     >>> class Example(object):
-    ...    @nmproperty:
+    ...    @nmevent.nmproperty
     ...    def x(self):
     ...       return self._x
-    ...
+    ...    
     ...    @x.setter
     ...    def x(self, value):
     ...       self._x = value
+    ...    
+    ...    x_changed = nmevent.Event()
+    ...    x.changed = x_changed
+    ...    
+    ...    def __init__(self):
+    ...       self._x = 0
+    ...
+    >>> def handler(sender, **keywords):
+    ...    print "x changed, old value: %r, new value: %r" % (keywords['old_value'], sender.x)
     ...
     >>> example = Example()
-    >>> exmaple.x_changed += handler
+    >>> example.x_changed += handler
     >>> example.x = 42
+    x changed, old value: 0, new value: 42
 
     .. attribute:: fget
        
@@ -460,11 +511,11 @@ class Property(object):
        ``INotifyPropertyChanged`` interface (see
        http://msdn.microsoft.com/en-US/library/system.componentmodel.inotifypropertychanged.aspx)
     """
-
+    
     @property
     def name(self):
         """The name of the property.
-
+        
         This should be the name of an object's attribute
         that holds the property. The name is guessed by
         retrieving the name of the getter function if present.
@@ -472,24 +523,24 @@ class Property(object):
         if self.fget is not None:
             return self.fget.__name__
         return None
-
+    
     def __init__(self, fget = None, fset = None, fdel = None,
                  changed = None, property_changed = None):
         """Constructor."""
         self.fget = fget
         self.fset = fset
         self.fdel = fdel
-
+        
         self.changed = changed
         self.property_changed = property_changed
-
+    
     def __get__(self, obj, objtype = None):
         if obj is None:
             return self
         if self.fget is None:
             raise AttributeError, "Unreadable attribute."
         return self.fget(obj)
-
+    
     def __set__(self, obj, value):
         if self.fset is None:
             raise AttributeError, "Can't set attribute."
@@ -499,25 +550,30 @@ class Property(object):
         old_value = self.fget(obj)
         self.fset(obj, value)
         if old_value != value:
-            if self.changed is not None:
-                self.changed(old_value = old_value)
-            if self.property_changed is not None:
-                self.property_changed(old_value = old_value, name = self.name)
-
+            self.fire_changed(obj.__class__, obj, old_value)
+    
     def __delete__(self, obj):
         if self.fdel is None:
             raise AttributeError, "Can't delete attribute."
         self.fdel(obj)
-
+    
+    def fire_changed(self, objtype, obj, old_value):
+        changed = self.changed and self.changed.bind(objtype, obj)
+        if changed:
+            changed(old_value = old_value)
+        property_changed = self.property_changed and self.property_changed.bind(objtype, obj)
+        if self.property_changed:
+            property_changed(old_value = old_value, name = self.name)
+    
     def setter(self, function):
         """Sets the setter function and returns self.
-
+        
         This function is meant to be used as a method decorator,
         even though it can be called directly to set the
         setter function of its property.
-
+        
         Usage:
-
+        
         >>> class ExampleClass(object):
         ...    def x(self):
         ...       return self._x
@@ -534,7 +590,7 @@ class Property(object):
         ...    # Also works, but looks ugly.
         ...    x.setter(set_x)
         ...
-
+        
         :param function: the property setter function
         :returns: self
         """
@@ -551,39 +607,46 @@ class Property(object):
         """
         self.fdel = function
         return self
-
+    
 def nmproperty(function):
     """Eventful property decorator.
-
+    
     Creates new :class:`Property` object using the decorated method
     as the getter function. Setter and deleter functions can be
     set by the :meth:`Property.setter` and :meth:`Property.deleter`
     decorators.
-
+    
     This decorator is called :func:`nmproperty` to avoid name conflict
     with the built-in `property` function and decorator.
-
+    
     Usage:
-
+    
     >>> class ExampleClass(object):
     ...    @nmevent.nmproperty
     ...    def x(self):
     ...       return self._x
     ...    
     ...    @x.setter
-    ...    def x(self, value)
+    ...    def x(self, value):
     ...       self._x = value
     ...
-    ...    x_changed = EventSlot
+    ...    x_changed = nmevent.Event()
     ...    x.changed = x_changed
     ...
+    ...    def __init__(self):
+    ...       self._x = None
+    ...
+    >>> def handler(sender, **keywords):
+    ...    print "handler called"
+    ...
     >>> example = ExampleClass()
     >>> example.x_changed += handler # "handler" will be called when the value of x changes
-    >>> example.x = 10 # value of x changed, "handler" should get called
-
+    >>> example.x = 10 # value of x changed, "handler" gets called
+    handler called
+    
     The :attr:`Property.changed` events can be automatically created and set
     by the :func:`with_events` decorator when used on the class.
-
+    
     :param function: function to be used as the property getter function
     :returns: new `Property` object
     """
@@ -608,25 +671,27 @@ def with_events(clss):
     ...
     ...    @x.setter
     ...    def x(self, value):
-    ...       self._x
+    ...       self._x = value
     ...
     ...    def __init__(self):
     ...       self._x = 0
     ...
-    >>> def x_changed_handler(self, sender, **keywords):
+    >>> def x_changed_handler(sender, **keywords):
     ...    old_value = keywords['old_value']
     ...    print "x changed; %r -> %r" % (old_value, sender.x)
     ...
-    >>> def property_changed_handler(self, sender, **keywords):
+    >>> def property_changed_handler(sender, **keywords):
     ...    old_value = keywords['old_value']
     ...    name = keywords['name']
-    ...    print "property \"%s\" changed, %r -> %r" % (old_value, sender.x)
+    ...    print "property '%s' changed, %r -> %r" % (name, old_value, sender.x)
     ...
     >>> example = Example()
     >>> example.x_changed += x_changed_handler
-    >>> example.property_changed_handler += property_changed_handler
+    >>> example.property_changed += property_changed_handler
     >>> example.x = 42
-
+    x changed; 0 -> 42
+    property 'x' changed, 0 -> 42
+    
     In the example above, the :func:`with_events` decorator automatically
     decorates the class with an ``x_changed`` event and ``property_changed``
     event connects them to the instance of :class:`Property` class created by
@@ -637,17 +702,85 @@ def with_events(clss):
     ``x_changed`` gets called only when ``Example.x`` changes,
     ``property_changed`` gets called when any property changes.
     """
-
+    
     property_changed = Event()
     setattr(clss, "property_changed", property_changed)
 
     for name, attr in clss.__dict__.items():
+        changed_attr = "%s_changed" % name
         if isinstance(attr, __builtin__.property):
-            setattr(clss, name + "_changed", Event())
+            setattr(clss, changed_attr, Event())
         elif isinstance(attr, Property):
-            slot = Event()
-            setattr(clss, name + "_changed", slot)
-            attr.changed = slot
+            setattr(clss, changed_attr, Event())
+            # Use getattr to bind the event to the class.
+            attr.changed = getattr(clss, changed_attr)
             attr.property_changed = property_changed
     return clss
 
+def with_properties(clss):
+    """Decorates a class with automatic "private" attributes.
+    
+    :param clss: class object to decorate.
+    :returns:    decorated class
+    
+    For every :class:`Property` instance within the class'
+    dictionary, it creates a setter and getter (unless they
+    are already set to a non-None value). These setters
+    and getters use "private" attributes - attributes
+    that the same name as the property prepended with an
+    underscore. In other words, for a property ``foo``,
+    you get an attribute called ``_foo`` where the actual
+    value is stored.
+    
+    Usage:
+    
+    >>> @nmevent.with_properties
+    ... class Example(object):
+    ...     foo = nmevent.Property()
+    ...     foo_changed = nmevent.Event()
+    ...     foo.changed = foo_changed
+    ...
+    >>> def on_foo_changed(sender, old_value):
+    ...     print "foo changed"
+    ... 
+    >>> x = Example()
+    >>> x.foo_changed += on_foo_changed
+    >>> x.foo = 42 # on_foo_changed gets called
+    foo changed
+    
+    Used together with ``with_events``:
+    
+    >>> @nmevent.with_events
+    ... @nmevent.with_properties
+    ... class NextExample(object):
+    ...     bar = nmevent.Property()
+    ...
+    """
+    
+    def make_getter(attr):
+        def getter(self):
+            if not hasattr(self, attr):
+                setattr(self, attr, None)
+            return getattr(self, attr)
+        return getter
+
+    def make_setter(attr):
+        def setter(self, value):
+            setattr(self, attr, value)
+        return setter
+    
+    for name, attr in clss.__dict__.items():
+        if isinstance(attr, Property):
+            private_attr = "_%s" % name
+            if not attr.fget:
+                attr.fget = make_getter(private_attr)
+            if not attr.fset:
+                attr.fset = make_setter(private_attr)
+    return clss
+
+def decorated(clss):
+    """Convenience decorator, which simply combines the :func:`with_events`
+    and :func:`with_properties` decorators.
+    """
+    return with_events(with_properties(clss))
+
